# -*- perl -*-

require 5.8.0;
use strict;
use vars qw($DBI_INC_DIR);
require ExtUtils::MakeMaker;

eval { require 5.8.0 } or die << 'EOD';
#######################################
#	This module requires a minimum Perl version of 5.8.0
#	Please upgrade!
#######################################
EOD

print "

 *** Configuring DBD::Teradata ...
 *** NOTE: The optional CLI adapter module
 *** DBD::Teradata::Cli is installed separately.
 *** Refer to the INSTALL_CLI.txt file in the
 *** DBD::Teradata::Cli bundle for installation
 *** instructions.

";

my %opts = (
	'NAME' => 'DBD::Teradata',
	'VERSION_FROM' => 'lib/DBD/Teradata.pm',
	'dist'         => {
	 	'DIST_DEFAULT' => 'all tardist',
     	'SUFFIX'       => 'gz',
	 	'COMPRESS'     => 'gzip -9vf'
	 },

    'AUTHOR' => 'Presicient Corp. (darnold@presicient.com)',
    'ABSTRACT' => 'Perl DBI driver for Teradata',
    'PREREQ_PM' => {},
);

my $ok = 1;
my $failed = undef;

$ok = undef,
$failed = 1
	unless CheckModule('DBI', '1.40');

$ok = undef,
print '

	Math::BigInt is required for encrypted logons in pure Perl mode,
	and	to support large precision DECIMAL, as well as more accurate 
	DECIMAL conversions. Since Math::BigInt is not available, DECIMAL 
	values will be converted via floating point arithmetic, and large 
	precision DECIMAL and pure Perl encrypted logons will not be 
	supported.
'
	unless CheckModule('Math::BigInt', '1.89');

$ok = undef,
print '

	To support encrypted logons in pure Perl mode,
	you may need to install

	For V2R5.1 and V2R6.0 releases:
		Crypt::Blowfish or Crypt::Blowfish_PP

	For V2R6.1 releases:
		Crypt::Rijndael + Digest::SHA1 + Crypt::ECB
'
	unless (CheckModule('Crypt::Blowfish') ||
		CheckModule('Crypt::Blowfish_PP')) &&
		CheckModule('Crypt::Rijndael') &&
		CheckModule('Crypt::ECB') &&
		CheckModule('Digest::SHA1');

print '

	Missing modules are available from any CPAN mirror, e.g.,
	http://search.cpan.org/

'
	unless $ok;

&check_DBI();

exit(0) if $failed;

print "

Looks good, creating makefile...
";

ExtUtils::MakeMaker::WriteMakefile(%opts);

sub check_DBI {
    eval { require DBI::DBD };
    foreach (@INC) {
  	    $DBI_INC_DIR = "-I$_/auto/DBI",
	    last
        	if -f "$_/auto/DBI/DBIXS.h";
    }
    unless (defined($DBI_INC_DIR)) {
        print "Unable to locate auto/DBI/DBIXS.h in the following directories:\n\n";
   	    print "    $_\n"
        	foreach (@INC);

        print "\nDoesn't look like a correct DBI installation.\n";
        $failed = 1;
    }
	exit(0)
    	if $failed;
}

sub CheckModule ($$;$) {
    my($module, $version, $package) = @_;
    $package ||= $module;
    if ($version) {
		print "Checking for $package, $version or later ... ";
		eval "use $module $version";
	    print "\nYou don't have the $package package version $version or later, installed.\n" and
	    return 0
			if $@;
    }
    else {
		print "Checking for $package ... ";
		eval "use $module";
		print "\nYou don't have the $package package installed.\n" and
		return 0
			if $@;
    }
    print "ok\n";
    return 1;
}

