# -*- perl -*-

require 5.8.0;
use strict;
use vars qw($DBI_INC_DIR);
require ExtUtils::MakeMaker;

eval { require 5.8.0 } or die << 'EOD';
#######################################
#	This module requires a minimum Perl version of 5.8.0
#	Please upgrade!
#######################################
EOD

my ($inc, $libs, $defines) = ('','','');

if ($^O eq 'MSWin32') {
#
#	user must set the TDAT_DBD_CLI_LIB environment variable
#	first time they install on windows (or on re-install)
#	we'll default to common location
#
	unless (defined($ENV{TDAT_DBD_CLI_LIB})) {
		$ENV{TDAT_DBD_CLI_LIB} = -e 'C:\\PROGRA~1\\NCR\\TERADA~1\\cli\\lib' 
			? 'C:\\PROGRA~1\\NCR\\TERADA~1\\cli\\lib'
			: -e 'C:\\PROGRA~1\\NCR\\TERADA~2\\cliv2\\lib'
				? 'C:\\PROGRA~1\\NCR\\TERADA~2\\cliv2\\lib'
				: undef;
	}
	
	unless (defined($ENV{TDAT_DBD_CLI_INC})) {
		$ENV{TDAT_DBD_CLI_INC} = -e 'C:\\PROGRA~1\\NCR\\TERADA~1\\cli\\inc'
			? 'C:\\PROGRA~1\\NCR\\TERADA~1\\cli\\inc'
			: -e 'C:\\PROGRA~1\\NCR\\TERADA~2\\cliv2\\inc'
				? 'C:\\PROGRA~1\\NCR\\TERADA~2\\cliv2\\inc'
				: undef;
	}

	$inc = '-I' . $ENV{TDAT_DBD_CLI_INC};
	if (-e "$ENV{TDAT_DBD_CLI_LIB}\\wincli32.lib") {
		$libs .= "-L$ENV{TDAT_DBD_CLI_LIB} -lwincli32";
#		print $libs, "\n";
		foreach ('tdusr32', 'localisation', 'tdgss', 'terasso') {
			$libs .= " -l$_"
				if -e "$ENV{SystemRoot}\\System32\\$_.dll";
		}
	}
}
else {
#
#	how do we pick where the libs are installed ? for now assume /usr/lib
#
	my $libdir = $ENV{TDAT_DBD_CLI_LIB} ||= '/usr/lib';
	$inc = '-I' . $ENV{TDAT_DBD_CLI_INC}
		if defined($ENV{TDAT_DBD_CLI_INC});
	$defines = '-D__error_t_defined=1';
	if (-e "$libdir/libcliv2.so") {
		$libs .= "-L$libdir -lcliv2 -lnet -lsocket -lresolv";

		foreach ('tdusr', 'tdgss') {
			$libs .= " -l$_.so"
				if -e "$libdir/lib$_.so";
		}
	}
}

print "

 *** Configuring DBD::Teradata::Cli ...

***************************************************************
*
*	!!!NOTE TO INSTALLERS!!!
*
*	DBD::Teradata::Cli will be built using the following
*	directives:
*	Libraries: $libs
*	Include files: $inc
*	Compile flags: $defines
*
*	If your CLI2 libraries and/or include files are in another
*	location, please update the TDAT_DBD_CLI_LIB and
*	TDAT_DBD_CLI_INC environment variables before running
*	Makefile.PL.
*
***************************************************************

";

my %prereq;
my %opts = (
	'NAME' => 'DBD::Teradata::Cli',
	'VERSION_FROM' => 'lib/DBD/Teradata/Cli.pm',
	'dist'         => {
	 	'DIST_DEFAULT' => 'all tardist',
     	'SUFFIX'       => 'gz',
	 	'COMPRESS'     => 'gzip -9vf'
	 },

    'AUTHOR' => 'Dean Arnold (darnold@presicient.com)',
    'ABSTRACT' => 'CLI Adapter for DBD::Teradata',
    'PREREQ_PM' => \%prereq,
    'LIBS' => $libs,
    'INC' => $inc,
    'CCFLAGS' => $defines
);

my $ok = 1;
$ok &&= CheckModule('DBI', '1.40');

unless ($ok) {
    print "\n",<<'MSG';

	Missing modules are available from any http://search.cpan.org/

MSG

}

&check_DBI();

ExtUtils::MakeMaker::WriteMakefile(%opts);

sub check_DBI {
    my $dir;
    eval { require DBI::DBD };
    foreach $dir (@INC) {
        if (-f "$dir/auto/DBI/DBIXS.h") {
  	    $DBI_INC_DIR = "-I$dir/auto/DBI";
	    last;
        }
     }
    if (!defined($DBI_INC_DIR)) {
        print "Unable to locate auto/DBI/DBIXS.h in the following directories:\n";
        print "\n";
        foreach $dir (@INC) {
    	    print "    $dir\n";
        }
        print "\nDoesn't look like a correct DBI installation.\n";
        $ok = 0;
    }
    if (!$ok) {
        exit(0);
    }
}

sub CheckModule ($$;$) {
    my($module, $version, $package) = @_;
    $package ||= $module;
    $prereq{$module} = $version || 0;
    if ($version) {
		print "Checking for $package, $version or later ... ";
		eval "use $module $version";
		if ($@) {
		    print("\nYou don't have the $package package",
			  " version $version or later, installed.\n");
		    return 0;
		}
    } else {
		print "Checking for $package ... ";
		eval "use $module";
		if ($@) {
		    print("\nYou don't have the $package package installed.\n");
		    return 0;
		}
    }
    print "ok\n";
    return 1;
}
